@@ -11,6 +11,7 @@ from pydantic import BaseModel
 from toolz import keyfilter
 
 API_SITE_ID = os.getenv("API_SITE_ID")
+API_SITE_CLIENT_ID = os.getenv("API_SITE_CLIENT_ID")
 
 
 def omit(blacklist, d):
@@ -27,10 +28,13 @@ def hash_from_str(s: str, algo="sha256") -> str:
     return getattr(hashlib, algo)(s.encode("utf-8")).hexdigest()
 
 
-def sha256hash_from_file(file_path: str):
+def sha256hash_from_file(file_path: str, timestamp: str):
     # https://stackoverflow.com/a/55542529
     h = hashlib.sha256()
 
+    timestamp_bytes = timestamp.encode("utf-8")
+    h.update(timestamp_bytes)
+
     with open(file_path, "rb") as file:
         while True:
             # Reading is buffered, so we can read smaller chunks.
@@ -81,7 +85,7 @@ def generate_secret(length=12):
     """
     if length < 8:
         raise ValueError(f"{length=} must be >=8.")
-    alphabet = string.ascii_letters + string.digits + string.punctuation
+    alphabet = string.ascii_letters + string.digits + "!@#$%^*-_+="
     # based on https://docs.python.org/3.8/library/secrets.html#recipes-and-best-practices
     while True:
         _secret = "".join(secrets.choice(alphabet) for i in range(length))
@@ -97,6 +101,6 @@ def generate_secret(length=12):
 def json_clean(data, model, exclude_unset=False) -> dict:
     """Run data through a JSON serializer for a pydantic model."""
     if not isinstance(data, (dict, BaseModel)):
-        raise TypeError("`data` must be a pydantic model or its .dict()")
+        raise TypeError("`data` must be a pydantic model or its .model_dump()")
     m = model(**data) if isinstance(data, dict) else data
     return json.loads(m.json(exclude_unset=exclude_unset))
