@@ -1,5 +1,5 @@
 import datetime
-from typing import Optional, Dict, Any
+from typing import Optional, Dict, Any, List
 
 from pydantic import BaseModel
 
@@ -9,14 +9,20 @@ from nmdc_runtime.api.models.workflow import Workflow
 
 class JobBase(BaseModel):
     workflow: Workflow
-    name: Optional[str]
-    description: Optional[str]
+    name: Optional[str] = None
+    description: Optional[str] = None
+
+
+class JobClaim(BaseModel):
+    op_id: str
+    site_id: str
 
 
 class Job(JobBase):
     id: str
-    created_at: Optional[datetime.datetime]
+    created_at: Optional[datetime.datetime] = None
     config: Dict[str, Any]
+    claims: List[JobClaim] = []
 
 
 class JobExecution(BaseModel):
