@@ -7,8 +7,8 @@ from nmdc_runtime.api.models.object import DrsObject
 
 
 class ObjectTypeBase(BaseModel):
-    name: Optional[str]
-    description: Optional[str]
+    name: Optional[str] = None
+    description: Optional[str] = None
 
 
 class ObjectType(ObjectTypeBase):
@@ -17,4 +17,4 @@ class ObjectType(ObjectTypeBase):
 
 
 class DrsObjectWithTypes(DrsObject):
-    types: Optional[List[str]]
+    types: Optional[List[str]] = None
